from .grpc_lib.darsman_notification_pb2 import MessagingRequest
from .grpc_lib.darsman_notification_pb2_grpc import (
    MessagingControllerStub,
)

CONTROLLER_STUB = MessagingControllerStub()
REQUEST_MESSAGE = MessagingRequest()
RPC_METHOD_NAME = "SendMessage"
SUCCESS_CODE = "0"


class NotificationTypes:
    SMS_NOTIFICATION = "sms"
    EMAIL_NOTIFICATION = "email"


class EventProTypes:
    LOGIN = "login"
    REGISTER = "register"
    WELCOME = "welcome"
    DISCOUNT_1MO_AFTER_REG = "discount_1mo_after_reg"
    HAPPY_BIRTHDAY = "hbd"
    CERTIFICATE_SUBMIT = "cert_submit"
    CERTIFICATE_EDIT = "cert_edit"
    MASTER_COURSE_INFORM = "edu_sup_inform"
    INITIAL_REGISTER = "init_course_reg"
    ROAD_MAP = "roadmap"
    CONSULT_REQUEST = "success_consult_req"
    DISCOUNT = "discount"
    MASTER_COURSE_REGISTER_FORM = "master_course_register_form"
