# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from . import darsman_notification_pb2


class MessagingControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            self.channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.SendMessage = self.channel.unary_unary(
                "/config.darsman_notification.MessagingController/SendMessage",
                request_serializer=darsman_notification_pb2.MessagingRequest.SerializeToString,
                response_deserializer=darsman_notification_pb2.MessagingResponse.FromString,
            )


class MessagingControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SendMessage(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_MessagingControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "SendMessage": grpc.unary_unary_rpc_method_handler(
            servicer.SendMessage,
            request_deserializer=darsman_notification_pb2.MessagingRequest.FromString,
            response_serializer=darsman_notification_pb2.MessagingResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.darsman_notification.MessagingController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class MessagingController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SendMessage(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.darsman_notification.MessagingController/SendMessage",
            darsman_notification_pb2.MessagingRequest.SerializeToString,
            darsman_notification_pb2.MessagingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
