# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: darsman_notification/grpc/darsman_notification.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n4darsman_notification/grpc/darsman_notification.proto\x12\x1b\x63onfig.darsman_notification\x1a\x1cgoogle/protobuf/struct.proto"\x9a\x01\n\x10MessagingRequest\x12\x13\n\x0bsender_unit\x18\x01 \x01(\t\x12\x11\n\trecipient\x18\x03 \x01(\t\x12\x12\n\nextra_data\x18\x04 \x01(\t\x12\x10\n\x08pro_type\x18\x05 \x01(\t\x12\x11\n\tsend_type\x18\x06 \x01(\t\x12%\n\x04\x64\x61ta\x18\x07 \x01(\x0b\x32\x17.google.protobuf.Struct"@\n\x11MessagingResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\t2\x85\x01\n\x13MessagingController\x12n\n\x0bSendMessage\x12-.config.darsman_notification.MessagingRequest\x1a..config.darsman_notification.MessagingResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "darsman_notification.grpc.darsman_notification_pb2", _globals
)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_MESSAGINGREQUEST"]._serialized_start = 116
    _globals["_MESSAGINGREQUEST"]._serialized_end = 270
    _globals["_MESSAGINGRESPONSE"]._serialized_start = 272
    _globals["_MESSAGINGRESPONSE"]._serialized_end = 336
    _globals["_MESSAGINGCONTROLLER"]._serialized_start = 339
    _globals["_MESSAGINGCONTROLLER"]._serialized_end = 472
# @@protoc_insertion_point(module_scope)
